@@ -149,6 +149,22 @@ up_backend_coldplug (UpBackend *backend, UpDaemon *daemon)
 	return TRUE;
 }
 
+/**
+ * up_backend_unplug:
+ * @backend: The %UpBackend class instance
+ *
+ * Forget about all learned devices, effectively undoing up_backend_coldplug.
+ * Resources are released without emitting signals.
+ */
+void
+up_backend_unplug (UpBackend *backend)
+{
+	if (backend->priv->daemon != NULL) {
+		g_object_unref (backend->priv->daemon);
+		backend->priv->daemon = NULL;
+	}
+}
+
 /**
  * up_backend_get_critical_action:
  * @backend: The %UpBackend class instance
@@ -399,7 +415,6 @@ static gboolean
 up_apm_device_refresh(UpDevice* device)
 {
 	UpDeviceKind type;
-	GTimeVal timeval;
 	gboolean ret;
 	g_object_get (device, "type", &type, NULL);
 
@@ -415,10 +430,8 @@ up_apm_device_refresh(UpDevice* device)
 			break;
 	}
 
-	if (ret) {
-		g_get_current_time (&timeval);
-		g_object_set (device, "update-time", (guint64) timeval.tv_sec, NULL);
-	}
+	if (ret)
+		g_object_set (device, "update-time", (guint64) g_get_real_time () / G_USEC_PER_SEC, NULL);
 
 	return ret;
 }
@@ -578,8 +591,8 @@ static void
 up_backend_init (UpBackend *backend)
 {
 	GError *err = NULL;
-	GTimeVal timeval;
 	UpDeviceClass *device_class;
+	gint64 current_time;
 
 	backend->priv = UP_BACKEND_GET_PRIVATE (backend);
 	backend->priv->is_laptop = up_native_is_laptop();
@@ -604,7 +617,7 @@ up_backend_init (UpBackend *backend)
 		}
 
 		/* setup dummy */
-		g_get_current_time (&timeval);
+		current_time = g_get_real_time () / G_USEC_PER_SEC;
 		g_object_set (backend->priv->battery,
 			      "type", UP_DEVICE_KIND_BATTERY,
 			      "power-supply", TRUE,
@@ -614,13 +627,13 @@ up_backend_init (UpBackend *backend)
 			      "state", UP_DEVICE_STATE_UNKNOWN,
 			      "percentage", 0.0f,
 			      "time-to-empty", (gint64) 0,
-			      "update-time", (guint64) timeval.tv_sec,
+			      "update-time", (guint64) current_time,
 			      (void*) NULL);
 		g_object_set (backend->priv->ac,
 			      "type", UP_DEVICE_KIND_LINE_POWER,
 			      "online", TRUE,
 			      "power-supply", TRUE,
-			      "update-time", (guint64) timeval.tv_sec,
+			      "update-time", (guint64) current_time,
 			      (void*) NULL);
 	}
 }
